<?php

namespace TecDiet\Util;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class CRUD{

	/**
	 * @var Connection connection
	 */
	protected $connection;
	/**
	 * @var Connection connectionParams;
	 */
	protected $connectionParams;
	/**
	 * @var EntityManager entityManager
	 */
	protected $entityManager;
	
	/**
	 * @const string PrimaryKeys
	 */
	const GET_PRIMARY_KEYS = "SELECT COLUMN_NAME FROM DBA_CONS_COLUMNS WHERE UPPER(OWNER) = UPPER(:OWNER) AND UPPER(TABLE_NAME) = UPPER(:TABLE_NAME) AND UPPER(CONSTRAINT_NAME) = (SELECT CONSTRAINT_NAME FROM DBA_CONSTRAINTS WHERE UPPER(TABLE_NAME) = UPPER(:TABLE_NAME) AND CONSTRAINT_TYPE = 'P' AND UPPER(OWNER) = UPPER(:OWNER))";
	
	/**
     * Construct...
     *
     * @param EntityManager $entityManager Used to create Connection
     */
	public function __construct($entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->connectionParams = $entityManager->getConnection()->getParams();
    }
    
    /** Public methods **/
    
	/**
     * Execute query update by params
     *
     * @param string query
     * @param array params
     * 
     */
	public function update($query, $params){
		$this->_executeUpdate($query, $params);
	}
	/**
     * Execute query find(select) by params
     *
     * @param string query 
     * @param array params
     * 
     */
	public function read($query, $params){
		return $this->_executeSelect($query, $params);
	}
	/**
     * Delete rows by primaryKeys
     *
     * @param string table name 
     * @param array rows
     * 
     */
    public function delete($tableName, $rows){
		$defaultQuery  = "DELETE FROM ".$tableName;
		$primaryKeys = $this->_getPrimaryKeys($this->connectionParams['user'], $tableName);
    	$query  = $this->_buildQueryConditions($defaultQuery, $primaryKeys);
    	$data = $this->_buildDataSetIfRow($rows);
    	
    	//Does not allow full printing of tables
    	if($defaultQuery !== $query){
			foreach ($data as $row) {
				$params = [];
		    	$params = $this->_buildQueryParams($primaryKeys, $row);
				$this->_executeDelete($query, $params);
		    }
    	}
	}
	
	/**
     * Create params and call procedure by name.
     *
     * @param string procedure name
     * @param array params
     * 
     * @return callback procedure
     * 
     * Exemplo:
	 * $params = array('PARAM1' => array('PARAM1' => '0002',
	 *							         'OUTPUT'  => true),
	 *		           'PARAM2' => '357-9');
	 * CRUD->callProcedure('MY_PROCEDURE_NAME', $params);
	 */
	public function callProcedure($procedureName, $params){
		
		$procedure = new StoredProcedure($this->connection, $procedureName);
        foreach ($params as $key => $value) {
			// if(is_array($value) && array_key_exists('OUTPUT', $value) && $value['OUTPUT']){
			if(is_array($value) && \Zeedhi\Framework\Util\Functions::arrayKeyExists('OUTPUT', $value) && $value['OUTPUT']){
    			$procedure->addParam(new Param($key, Param::PARAM_OUTPUT, $value[$key]));
			} else {
        		if(is_array($value)){
        			$procedure->addParam(new Param($key, Param::PARAM_INPUT, $value[$key]));
				} else {
        			$procedure->addParam(new Param($key, Param::PARAM_INPUT, $value));
				}
			}
		}
        
        $callback = $procedure->execute();
		//clear params
		$this->params = array();
        return $callback;
	}
	
	/** Private methods **/
	
	/**
     * Build the query according to the array of primary keys
     *
     * @param string query 
     * @param array primaryKeys
     * 
     * @return string complete query 
     */
    private function _buildQueryConditions($query, $primaryKeys) {
		$queryParams = [];
		$cndtPrmbl = ' WHERE';
		$operator = "=";
		if($primaryKeys && !empty($primaryKeys)){
			foreach ($primaryKeys as $column) {
				$query = $query.$cndtPrmbl." {$column} {$operator} :{$column}";
	    		$cndtPrmbl = ' AND';
			}
		}
    	return $query;
    }
	
	/**
     * Build params array by rows 
     *
     * @param array primaryKeys 
     * @param array rows
     * 
     * @return array params 
     */
	private function _buildQueryParams($primaryKeys, $rows){
    	$params = [];
    	foreach ($primaryKeys as $key) {
    		$keyLower = strtolower($key);
    		if (gettype($keyLower) != 'integer') {
    			$params[$keyLower] = $rows[$keyLower];
    		}
    	}
    	return $params;
    }
	
	/**
     * Get as primary keys from the table passed as parameter 
     *
     * @param string owner 
     * @param string tableName
     * 
     * @return array primaryKeys
     */
	private function _getPrimaryKeys($owner, $tableName) {
		$primaryKeys = [];
        $result = $this->_executeSelect(self::GET_PRIMARY_KEYS, array(
            "OWNER" => $owner,
            "TABLE_NAME" => $tableName
        ));
        
        if(!empty($result))
            foreach($result as $primaryKey) {
                $primaryKeys[] = strtolower($primaryKey['COLUMN_NAME']);
            }
        return $primaryKeys;
    }
    
    /**
     * Build array of rows to delete.
     *
     * @param array rows 
     * 
     * @return array data
     */
    private function _buildDataSetIfRow($rows){
    	$data = [];
    	if(is_array($rows)){
    		foreach ($rows as $row) {
    			if(is_array($row)){
    				array_push($data, $row);
    			}
    		}
    		if(empty($data)){
    			array_push($data, $rows);
    		}
    	} else {
    		$data = [$rows];
    	}
    	return $data;
    }
    
	/**
     * Executes a select query for the parameters 
     *
     * @param string select
     * @param array params
     * 
     * @return fetchAll The result of the filter criteria.
     */
	private function _executeSelect($query, $params) {
	    $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(\PDO::FETCH_ASSOC);
	}
	
	/**
     * Executes a update query for the parameters 
     *
     * @param string update
     * @param array params
     */
	private function _executeUpdate($query, $params){
		$sql = $this->connection->prepare($query);
		$sql->execute($params);
	}
	
	/**	
     * Executes a delete query for the parameters 
     *
     * @param string query
     * @param string params
     */
	private function _executeDelete($query, $params){
		$sql = $this->connection->prepare($query);
		$sql->execute($params);
	}
}